using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

using RSSReader.RSS;

namespace RSSReader.Controls
{
	/// <summary>
	/// Summary description for SearchResultsView.
	/// </summary>
	public class SearchResultsView : System.Windows.Forms.UserControl
	{

		#region Public events 

		public delegate void ItemSelectedHandler( object sender, RSSItem item );
		public event ItemSelectedHandler ItemSelected;

		#endregion 

		#region 

		private System.Windows.Forms.ListView lvwItems;
		private System.Windows.Forms.ColumnHeader chItemTitle;
		private System.Windows.Forms.ColumnHeader chItemDate;
		private System.Windows.Forms.ColumnHeader chItemAuthor;
		private System.Windows.Forms.ColumnHeader chItemSubject;
		private System.Windows.Forms.ColumnHeader chRank;
		private System.Windows.Forms.ColumnHeader chFeed;
		private System.Windows.Forms.ImageList imageList1;
		private System.ComponentModel.IContainer components;

		public SearchResultsView()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			// TODO: Add any initialization after the InitializeComponent call

		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(SearchResultsView));
			this.lvwItems = new System.Windows.Forms.ListView();
			this.chItemTitle = new System.Windows.Forms.ColumnHeader();
			this.chItemDate = new System.Windows.Forms.ColumnHeader();
			this.chItemAuthor = new System.Windows.Forms.ColumnHeader();
			this.chItemSubject = new System.Windows.Forms.ColumnHeader();
			this.chRank = new System.Windows.Forms.ColumnHeader();
			this.chFeed = new System.Windows.Forms.ColumnHeader();
			this.imageList1 = new System.Windows.Forms.ImageList(this.components);
			this.SuspendLayout();
			// 
			// lvwItems
			// 
			this.lvwItems.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.lvwItems.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																					   this.chRank,
																					   this.chFeed,
																					   this.chItemTitle,
																					   this.chItemDate,
																					   this.chItemAuthor,
																					   this.chItemSubject});
			this.lvwItems.FullRowSelect = true;
			this.lvwItems.HideSelection = false;
			this.lvwItems.LabelWrap = false;
			this.lvwItems.Location = new System.Drawing.Point(0, 0);
			this.lvwItems.MultiSelect = false;
			this.lvwItems.Name = "lvwItems";
			this.lvwItems.Size = new System.Drawing.Size(510, 318);
			this.lvwItems.SmallImageList = this.imageList1;
			this.lvwItems.TabIndex = 8;
			this.lvwItems.View = System.Windows.Forms.View.Details;
			// 
			// chItemTitle
			// 
			this.chItemTitle.Text = "Title";
			this.chItemTitle.Width = 256;
			// 
			// chItemDate
			// 
			this.chItemDate.Text = "Date";
			this.chItemDate.Width = 117;
			// 
			// chItemAuthor
			// 
			this.chItemAuthor.Text = "Author";
			this.chItemAuthor.Width = 120;
			// 
			// chItemSubject
			// 
			this.chItemSubject.Text = "Subject";
			this.chItemSubject.Width = 181;
			// 
			// chRank
			// 
			this.chRank.Text = "Rank";
			// 
			// chFeed
			// 
			this.chFeed.Text = "Feed";
			this.chFeed.Width = 147;
			// 
			// imageList1
			// 
			this.imageList1.ImageSize = new System.Drawing.Size(16, 16);
			this.imageList1.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList1.ImageStream")));
			this.imageList1.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// SearchResultsView
			// 
			this.Controls.Add(this.lvwItems);
			this.Name = "SearchResultsView";
			this.Size = new System.Drawing.Size(520, 330);
			this.Load += new System.EventHandler(this.SearchResults_Load);
			this.ResumeLayout(false);

		}
		#endregion

		#endregion 

		#region Private variables 

		private ListViewSortHelper	sorter;
		private ThreadListItem		contextItem;
		private FeedSearchResults	results;

		#endregion 

		public FeedSearchResults SearchResults
		{

			get
			{
				return results;
			}

			set
			{

				results = value;

				this.lvwItems.Items.Clear();

				results.Sort();

				int rank = 1;
				foreach( FeedSearchResult result in results.Results )
				{
					RSSItem feedItem = (RSSItem) result.SearchResult;
					ShowFeedItem( rank, feedItem, null );
					rank += 1;
				}

			}

		}

		private void SearchResults_Load(object sender, System.EventArgs e)
		{
			
			this.lvwItems.Dock = DockStyle.Fill;
			this.lvwItems.DoubleClick += new EventHandler(lvwItems_DoubleClick);
			this.lvwItems.MouseDown += new MouseEventHandler(lvwItems_MouseDown);

			sorter = new ListViewSortHelper( this.lvwItems );
			sorter.SetColumnComparer( 0, new IntColumnComparer() );
			sorter.SetColumnComparer( 3, new DateColumnComparer() );
			sorter.SortOnColumn( 0 );

		}

		private void ShowFeedItem( int rank, RSSItem feedItem, ThreadListItem parent )
		{

			ThreadListItem item = CreateListItemFromRSSItem( rank, feedItem );

			if( parent == null )
				lvwItems.Items.Add( item );
			else
				parent.Items.Add( item );

		}

		private ThreadListItem CreateListItemFromRSSItem( int rank, RSSItem feedItem )
		{

			string title = System.Web.HttpUtility.HtmlDecode( feedItem.Title );

			ThreadListItem item = new ThreadListItem( rank.ToString() );
			item.SubItems.Add( System.Web.HttpUtility.HtmlDecode( feedItem.Parent.Title ) );
			item.SubItems.Add( title );
			item.SubItems.Add( feedItem.DatePublished.ToShortDateString() );
			item.SubItems.Add( feedItem.Creator );
			item.SubItems.Add( feedItem.Subject );
			item.Tag = feedItem;

			return item;
		
		}

		private void lvwItems_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
		
			contextItem = (ThreadListItem) lvwItems.GetItemAt( e.X, e.Y );

			if( e.Button != MouseButtons.Left )
				return;

			Point hitPoint = new Point( e.X, e.Y );
			ListViewItem stateClicked = ListViewHelper.StateImageHitTest( lvwItems, hitPoint );
			if( stateClicked != null )
			{

				stateClicked.Selected = true;

				if( stateClicked is ThreadListItem )
				{
					
					ThreadListItem threaded = (ThreadListItem) stateClicked;
					if( threaded.HasChildItems )
						threaded.Expanded = !threaded.Expanded;

					if( threaded.Expanded )
						threaded.StateImageIndex = 1;
					else if( threaded.HasChildItems )
						threaded.StateImageIndex = 0;

				}

			}

		}

		private void lvwItems_DoubleClick(object sender, EventArgs e)
		{

			if( contextItem == null )
				return;

			RSS.RSSItem item = (RSS.RSSItem) contextItem.Tag;
			if( this.ItemSelected != null )
				this.ItemSelected( this, item );

		}

	}

}
