using System;
using System.Collections;
using System.Windows.Forms;
    
namespace RSSReader.Controls
{
    
    /// <summary>
    /// Represents a collection of <see cref='RSSReader.Controls.ListViewItem'/> objects.
    /// </summary>
    // This class was created on 1/15/2005 by the SharpTools CollectionBuilder Addin.
    public class ListViewItemCollection : System.Collections.ICollection
    {

		#region Public events 
        
		/// <summary>
		/// Called when an object is being inserted into the collection.
		/// </summary>
		public event ListViewItemCollectionChangedEventHandler Inserting;
        
		/// <summary>
		/// Called after an object has been inserted into the collection.
		/// </summary>
		public event ListViewItemCollectionChangedEventHandler InsertComplete;
        
		/// <summary>
		/// Called when an object is being removed from the collection.
		/// </summary>
		public event ListViewItemCollectionChangedEventHandler Removing;
        
		/// <summary>
		/// Called after an object has been removed from the collection.
		/// </summary>
		public event ListViewItemCollectionChangedEventHandler RemoveComplete;
        
		/// <summary>
		/// Called when the collection is being cleared.
		/// </summary>
		public event ListViewItemCollectionClearedEventHandler Clearing;
        
		/// <summary>
		/// Called after the collection has been cleared.
		/// </summary>
		public event ListViewItemCollectionClearedEventHandler ClearComplete;

		/// <summary>
		/// Defines the type of event that is called whenever a new object is inserted into or removed from the RSSReader.Controls.ListViewItemCollection collection.
		/// </summary>
		public delegate void ListViewItemCollectionChangedEventHandler(object sender, ListViewItem value);
        
		/// <summary>
		/// Defines the type of event that is called whenever the collection is cleared.
		/// </summary>
		public delegate void ListViewItemCollectionClearedEventHandler(object sender);

		#endregion 
        
		#region Private member variables 
        
        // Provides the base object storage used by this class
        private System.Collections.ArrayList items;
        
        // This version counter is incremented any time the collection changes.
        private int listVersion;

		#endregion 

		#region Constructor 
        
        /// <summary>
        /// Initializes a new instance of the <see cref='RSSReader.Controls.ListViewItemCollection'/> class.
        /// </summary>
        public ListViewItemCollection()
        {
            items = new System.Collections.ArrayList();
            listVersion = 0;
        }

		#endregion 

		#region Internal properties 
        
        /// <summary>
        /// Returns a number that can be used to determine if the collection has changed since some previous
        /// reference point, since this value changes each time the collection is modified in any way.
        /// </summary>
        internal int Version
        {
            get
            {
                return listVersion;
            }
        }

		/// <summary>
		/// Internal method called whenever the collection changes to increment the Version property.
		/// </summary>
		private void IncrementVersion()
		{
			listVersion = (listVersion + 1);
		}
        
		#endregion 

		#region Public collection members 
        
        /// <summary>
        /// Gets the number objects contained in the collection.
        /// <summary>
        public virtual int Count
        {
            get
            {
                return items.Count;
            }
        }
        
        /// <summary>
        /// Gets a value indicating whether access to the collection is synchronized (thread-safe).
        /// <summary>
        public virtual bool IsSynchronized
        {
            get
            {
                return items.IsSynchronized;
            }
        }
        
        /// <summary>
        /// Gets an object that can be used to synchronize access to the collection.
        /// <summary>
        public virtual object SyncRoot
        {
            get
            {
                return items.SyncRoot;
            }
        }
        
        /// <summary>
        /// Returns the <see cref='RSSReader.Controls.ListViewItem'/> at the specified index of the <see cref='RSSReader.Controls.ListViewItemCollection'/>.
        /// </summary>
        /// <param name='index'>The zero-based index of the <see cref='RSSReader.Controls.ListViewItem'/> to locate in the collection.</param>
        /// <returns> The <see cref='RSSReader.Controls.ListViewItem'/> at the specified index of the collection.</returns>
        /// <exception cref='System.ArgumentOutOfRangeException'><paramref name='index'/> is outside the valid range of indexes for the collection.</exception>
        public virtual ListViewItem this[int index]
        {
            get
            {
                return ((ListViewItem)(this.items[index]));
            }
        }

        /// <summary>
        /// Returns a <see cref='System.Collections.IEnumerator'/> object that can be used to enumerate
        /// the entire collection.
        /// </summary>
        public virtual System.Collections.IEnumerator GetEnumerator()
        {
            return items.GetEnumerator();
        }
        
        /// <summary>
        /// Copies the elements of this RSSReader.Controls.ListViewItemCollection to a System.Array, starting at a particular System.Array index.
        /// <summary>
        public virtual void CopyTo(System.Array array, int index)
        {
            items.CopyTo(array, index);
        }
        
        /// <summary>
        /// Adds the specified <see cref='RSSReader.Controls.ListViewItem'/> to the <see cref='RSSReader.Controls.ListViewItemCollection'/> collection.
        /// </summary>
        /// <returns>The position into which the new element was inserted.</returns>
        public virtual int Add(ListViewItem value)
        {
            if ((this.Inserting != null))
            {
                this.Inserting(this, value);
            }
            this.IncrementVersion();
            int retVal = items.Add(value);
            if ((this.InsertComplete != null))
            {
                this.InsertComplete(this, value);
            }
            return retVal;
        }
        
        /// <summary>
        /// Removes the specified <see cref='RSSReader.Controls.ListViewItem'/> from the <see cref='RSSReader.Controls.ListViewItemCollection'/> collection.
        /// </summary>
        public virtual void Remove(ListViewItem value)
        {
            if ((this.Removing != null))
            {
                this.Removing(this, value);
            }
            this.IncrementVersion();
            items.Remove(value);
            if ((this.RemoveComplete != null))
            {
                this.RemoveComplete(this, value);
            }
        }
        
        /// <summary>
        /// Removes the <see cref='RSSReader.Controls.ListViewItem'/> object at the specified index from the <see cref='RSSReader.Controls.ListViewItemCollection'/> collection.
        /// </summary>
        public virtual void RemoveAt(int index)
        {
            object obj = items[index];
            this.Remove(((ListViewItem)(obj)));
        }
        
        /// <summary>
        /// Inserts a <see cref='RSSReader.Controls.ListViewItem'/> object into the <see cref='RSSReader.Controls.ListViewItemCollection'/> collection at the specified position.
        /// </summary>
        /// <param name='index'>The zero-based index at which value should be inserted.</param>
        /// <param name='value'>The RSSReader.Controls.ListViewItem to insert into the <see cref='RSSReader.Controls.ListViewItemCollection'/> collection.</param>
        public virtual void Insert(int index, ListViewItem value)
        {
            if ((this.Inserting != null))
            {
                this.Inserting(this, value);
            }
            this.IncrementVersion();
            items.Insert(index, value);
            if ((this.InsertComplete != null))
            {
                this.InsertComplete(this, value);
            }
        }
        
        /// <summary>
        /// determines whether the <see cref='RSSReader.Controls.ListViewItemCollection'/> collection contains a specific <see cref='RSSReader.Controls.ListViewItem'/> object.
        /// </summary>
        /// <param name='value'>The RSSReader.Controls.ListViewItem object to locate in the <see cref='RSSReader.Controls.ListViewItemCollection'/> collection.</param>
        /// <returns>True if the System.Object is found in the collection; otherwise, False.</returns>
        public virtual bool Contains(ListViewItem value)
        {
            return items.Contains(value);
        }
        
        /// <summary>
        /// Removes all items from the collection.
        /// </summary>
        public virtual void Clear()
        {
            if ((this.Clearing != null))
            {
                this.Clearing(this);
            }
            this.IncrementVersion();
            items.Clear();
            if ((this.ClearComplete != null))
            {
                this.ClearComplete(this);
            }
        }
        
        /// <summary>
        /// Creates a shallow copy of the collection.
        /// </summary>
        public virtual ListViewItemCollection Clone()
        {
            ListViewItemCollection obj = new ListViewItemCollection();
            obj.items = ((System.Collections.ArrayList)(items.Clone()));
            obj.listVersion = this.listVersion;
            return obj;
        }
        
		#endregion 

    }

}

